@@ -15,7 +15,11 @@ from urllib.parse import urljoin
 
 import httpx
 
-from async_firebase.base import AsyncClientBase
+from async_firebase.base import (  # noqa: F401
+    AsyncClientBase,
+    RequestLimits,
+    RequestTimeout,
+)
 from async_firebase.encoders import aps_encoder
 from async_firebase.messages import (
     AndroidConfig,
@@ -24,10 +28,10 @@ from async_firebase.messages import (
     APNSPayload,
     Aps,
     ApsAlert,
-    FcmPushMulticastResponse,
-    FcmPushResponse,
+    FCMBatchResponse,
+    FCMResponse,
     Message,
-    Notification,
+    MulticastMessage,
     PushNotification,
     WebpushConfig,
     WebpushFCMOptions,
@@ -35,14 +39,14 @@ from async_firebase.messages import (
     WebpushNotificationAction,
 )
 from async_firebase.utils import (
-    FcmPushMulticastResponseHandler,
-    FcmPushResponseHandler,
+    FCMBatchResponseHandler,
+    FCMResponseHandler,
     cleanup_firebase_message,
     serialize_mime_message,
 )
 
 DEFAULT_TTL = 604800
-MULTICAST_MESSAGE_MAX_DEVICE_TOKENS = 500
+BATCH_MAX_MESSAGES = MULTICAST_MESSAGE_MAX_DEVICE_TOKENS = 500
 
 
 class AsyncFirebaseClient(AsyncClientBase):
@@ -332,144 +336,136 @@ class AsyncFirebaseClient(AsyncClientBase):
             fcm_options=WebpushFCMOptions(link=link) if link else None,
         )
 
-    async def push(  # pylint: disable=too-many-locals
-        self,
-        device_token: str,
-        *,
-        android: t.Optional[AndroidConfig] = None,
-        apns: t.Optional[APNSConfig] = None,
-        condition: t.Optional[str] = None,
-        data: t.Optional[t.Dict[str, str]] = None,
-        notification: t.Optional[Notification] = None,
-        topic: t.Optional[str] = None,
-        webpush: t.Optional[WebpushConfig] = None,
-        dry_run: bool = False,
-    ) -> FcmPushResponse:
+    async def send(self, message: Message, *, dry_run: bool = False) -> FCMResponse:
         """
         Send push notification.
 
-        :param device_token: device token allows to send targeted notifications to a particular instance of app.
-        :param android: as instance of ``messages.AndroidConfig`` that contains Android-specific options.
-        :param apns: as instance of ``messages.APNSConfig`` that contains iOS-specific options.
-        :param condition: the Firebase condition to which the message should be sent.
-        :param data: a dictionary of data fields. All keys and values in the dictionary must be strings.
-        :param notification: an instance of ``messages.Notification`` that contains a notification that can be included
-            in a resulting message.
-        :param topic: name of the Firebase topic to which the message should be sent.
-        :param webpush: an instance of ``messages.WebpushConfig``.
+        :param message: the message that has to be sent.
         :param dry_run: indicating whether to run the operation in dry run mode (optional). Flag for testing the request
             without actually delivering the message. Default to ``False``.
 
         :raises:
 
-            ValueError is ``messages.PushNotification`` payload cannot be assembled
+            ValueError if ``messages.PushNotification`` payload cannot be assembled
 
-        :return: response from Firebase. Example of response:
+        :return: instance of ``messages.FCMResponse``
 
-            success::
+            Example of raw response:
 
-                {
-                    'name': 'projects/mobile-app/messages/0:1612788010922733%7606eb247606eb24'
-                }
+                success::
 
-            failure::
-
-                {
-                    'error': {
-                        'code': 400,
-                        'details': [
-                            {
-                                '@type': 'type.googleapis.com/google.rpc.BadRequest',
-                                'fieldViolations': [
-                                    {
-                                        'description': 'Value type for APS key [badge] is a number.',
-                                        'field': 'message.apns.payload.aps.badge'
-                                    }
-                                ]
-                            },
-                            {
-                                '@type': 'type.googleapis.com/google.firebase.fcm.v1.FcmError',
-                                'errorCode': 'INVALID_ARGUMENT'
-                            }
-                        ],
-                        'message': 'Value type for APS key [badge] is a number.',
-                        'status': 'INVALID_ARGUMENT'
+                    {
+                        'name': 'projects/mobile-app/messages/0:1612788010922733%7606eb247606eb24'
                     }
-                }
-        """
-        message = Message(
-            token=device_token,
-            data=data or {},
-            notification=notification,
-            android=android,
-            webpush=webpush,
-            apns=apns,
-            topic=topic,
-            condition=condition,
-        )
 
-        push_notification = self.assemble_push_notification(apns_config=apns, dry_run=dry_run, message=message)
+                failure::
+
+                    {
+                        'error': {
+                            'code': 400,
+                            'details': [
+                                {
+                                    '@type': 'type.googleapis.com/google.rpc.BadRequest',
+                                    'fieldViolations': [
+                                        {
+                                            'description': 'Value type for APS key [badge] is a number.',
+                                            'field': 'message.apns.payload.aps.badge'
+                                        }
+                                    ]
+                                },
+                                {
+                                    '@type': 'type.googleapis.com/google.firebase.fcm.v1.FcmError',
+                                    'errorCode': 'INVALID_ARGUMENT'
+                                }
+                            ],
+                            'message': 'Value type for APS key [badge] is a number.',
+                            'status': 'INVALID_ARGUMENT'
+                        }
+                    }
+        """
+        push_notification = self.assemble_push_notification(apns_config=message.apns, dry_run=dry_run, message=message)
 
         response = await self.send_request(
             uri=self.FCM_ENDPOINT.format(project_id=self._credentials.project_id),  # type: ignore
             json_payload=push_notification,
-            response_handler=FcmPushResponseHandler(),
+            response_handler=FCMResponseHandler(),
         )
-        if not isinstance(response, FcmPushResponse):
+        if not isinstance(response, FCMResponse):
             raise ValueError("Wrong return type, perhaps because of a response handler misuse.")
         return response
 
-    async def push_multicast(
+    async def send_multicast(
         self,
-        device_tokens: t.Union[t.List[str], t.Tuple[str]],
+        multicast_message: MulticastMessage,
         *,
-        android: t.Optional[AndroidConfig] = None,
-        apns: t.Optional[APNSConfig] = None,
-        data: t.Optional[t.Dict[str, str]] = None,
-        notification: t.Optional[Notification] = None,
-        webpush: t.Optional[WebpushConfig] = None,
         dry_run: bool = False,
-    ) -> FcmPushMulticastResponse:
+    ) -> FCMBatchResponse:
         """
         Send Multicast push notification.
 
-        :param device_tokens: the list of device tokens to send targeted notifications to a set of instances of app.
+        :param multicast_message: multicast message to send targeted notifications to a set of instances of app.
             May contain up to 500 device tokens.
-        :param android: as instance of ``messages.AndroidConfig`` that contains Android-specific options.
-        :param apns: as instance of ``messages.APNSConfig`` that contains iOS-specific options.
-        :param data: a dictionary of data fields. All keys and values in the dictionary must be strings.
-        :param notification: an instance of ``messages.Notification`` that contains a notification that can be included
-            in a resulting message.
-        :param webpush: an instance of ``messages.WebpushConfig``.
         :param dry_run: indicating whether to run the operation in dry run mode (optional). Flag for testing the request
             without actually delivering the message. Default to ``False``.
+
+        :raises:
+
+            ValueError if ``messages.PushNotification`` payload cannot be assembled
+            ValueError if ``messages.MulticastMessage`` contains more than MULTICAST_MESSAGE_MAX_DEVICE_TOKENS
+
+        :return: instance of ``messages.FCMBatchResponse``
         """
 
-        if len(device_tokens) > MULTICAST_MESSAGE_MAX_DEVICE_TOKENS:
+        if len(multicast_message.tokens) > MULTICAST_MESSAGE_MAX_DEVICE_TOKENS:
             raise ValueError(
                 f"A single ``messages.MulticastMessage`` may contain up to {MULTICAST_MESSAGE_MAX_DEVICE_TOKENS} "
                 "device tokens."
             )
 
+        messages = [
+            Message(
+                token=token,
+                data=multicast_message.data,
+                notification=multicast_message.notification,
+                android=multicast_message.android,
+                webpush=multicast_message.webpush,
+                apns=multicast_message.apns,
+            )
+            for token in multicast_message.tokens
+        ]
+
+        return await self.send_all(messages, dry_run=dry_run)
+
+    async def send_all(
+        self,
+        messages: t.Union[t.List[Message], t.Tuple[Message]],
+        *,
+        dry_run: bool = False,
+    ) -> FCMBatchResponse:
+        """
+        Send the given messages to FCM in a single batch.
+
+        :param messages: the list of messages to send.
+        :param dry_run: indicating whether to run the operation in dry run mode (optional). Flag for testing the request
+            without actually delivering the message. Default to ``False``.
+        :returns: instance of ``messages.FCMBatchResponse``
+        """
+
+        if len(messages) > BATCH_MAX_MESSAGES:
+            raise ValueError(f"A list of messages must not contain more than {BATCH_MAX_MESSAGES} elements")
+
         multipart_message = MIMEMultipart("mixed")
         # Message should not write out it's own headers.
         setattr(multipart_message, "_write_headers", lambda self: None)
 
-        for device_token in device_tokens:
+        for message in messages:
             msg = MIMENonMultipart("application", "http")
             msg["Content-Transfer-Encoding"] = "binary"
             msg["Content-ID"] = self.get_request_id()
             push_notification = self.assemble_push_notification(
-                apns_config=apns,
+                apns_config=message.apns,
                 dry_run=dry_run,
-                message=Message(
-                    token=device_token,
-                    data=data or {},
-                    notification=notification,
-                    android=android,
-                    webpush=webpush,
-                    apns=apns,
-                ),
+                message=message,
             )
             body = self.serialize_batch_request(
                 httpx.Request(
@@ -491,8 +487,8 @@ class AsyncFirebaseClient(AsyncClientBase):
             uri=self.FCM_BATCH_ENDPOINT,
             content=body,
             headers={"Content-Type": f"multipart/mixed; boundary={multipart_message.get_boundary()}"},
-            response_handler=FcmPushMulticastResponseHandler(),
+            response_handler=FCMBatchResponseHandler(),
         )
-        if not isinstance(batch_response, FcmPushMulticastResponse):
+        if not isinstance(batch_response, FCMBatchResponse):
             raise ValueError("Wrong return type, perhaps because of a response handler misuse.")
         return batch_response
